#include "StdAfx.h"
#include "DecodeSOF.h"

CDecodeSOF::CDecodeSOF()
{
	m_pTerrain	= NULL;
}

CDecodeSOF::~CDecodeSOF()
{
}

// Save
bool CDecodeSOF::SaveSOF(const wchar_t* pFileName, CTerrain* pTerrain)
{
	m_pTerrain = pTerrain;

	FILE *fp = _wfopen(pFileName, L"wb");
	if(fp == NULL)
	{
		return false;
	}

	// 
	wchar_t fileInfo[16] = L"SOFMAPFILE V1.0";
	SaveFileInfo(fp, fileInfo);		// 1.  
	SaveMapInfo(fp);					// 2.  
	SaveTileInfo(fp);					// 3. ŸϺ 
	SaveTexture(fp);					// 4. ؽ 
	SaveAlphaTexture(fp, pFileName);	// 5. ؽ 

	fclose(fp);
	m_pTerrain = NULL;

	return true;
}

void CDecodeSOF::SaveFileInfo(FILE* fp, wchar_t* pFileInfo)
{
	// [1]  
	fwrite(pFileInfo, 16, 1, fp);
}

void CDecodeSOF::SaveMapInfo(FILE* fp)
{
	// [2]  
	fwrite(&m_pTerrain->m_MapInfo, sizeof(MAPINFO), 1, fp);
}

void CDecodeSOF::SaveTileInfo(FILE* fp)
{
    int i=0;
	int totalVertex = m_pTerrain->m_MapInfo.nTotalVertices * m_pTerrain->m_MapInfo.nTotalVertices;
	float* temp = new float[totalVertex];
	for(i=0; i<totalVertex; ++i)
		temp[i] = m_pTerrain->m_pHeightMap[i].p.y;

	// [3] ü ؽ y
	fwrite(temp, sizeof(float), totalVertex, fp);
	delete [] temp;
}

void CDecodeSOF::SaveTexture(FILE* fp)
{
	int i=0;
	int selectTextureNum = m_pTerrain->m_nSelectTextureNum;

	// [5]  ؽ 
	fwrite(&selectTextureNum, sizeof(int), 1, fp);

    for(i=0; i<selectTextureNum; ++i)
	{
		// [6]  ؽ ȣ
		int index = m_pTerrain->m_nTextureIndex[i];
		fwrite(&index, sizeof(int) , 1, fp);

		// [*]  ؽ ̸(Ŭ̾Ʈ ÿ)
	//	char* texName = (char*)m_pTerrain->m_strTextureName[index].c_str();
	//	fwrite(texName, sizeof(64), 1, fp);
	}
}

void CDecodeSOF::SaveAlphaTexture(FILE* fp, const wchar_t* pFileName)
{
	int i=0;
	int selectTextureNum = m_pTerrain->m_nSelectTextureNum;
	int alphaSize = m_pTerrain->m_MapInfo.nAlphaTexSize*m_pTerrain->m_MapInfo.nAlphaTexSize;

	BYTE* read = new BYTE[alphaSize];
	for(i=0; i<selectTextureNum-1; ++i)
	{
		// [7]  ؽİ ǰ ִٸ İ Ѵ.
		m_pTerrain->SaveAlphaValue(i, read);
		fwrite(read, sizeof(BYTE), alphaSize, fp);
	}

	DELS(read);

	// [8]  ÷ Ǵ Ÿ  Ѵ.
	int tileNum = (int)m_pTerrain->m_vecSplattingTile.size();
	fwrite(&tileNum, sizeof(int), 1, fp);

	// [9]  ÷ Ǵ Ÿ ȣ Ѵ.
	unsigned int j=0;
	for(j=0; j<m_pTerrain->m_vecSplattingTile.size(); ++j)
		fwrite(&m_pTerrain->m_vecSplattingTile[j], sizeof(int), 1, fp);
}


// Load
bool CDecodeSOF::LodeSOF(const wchar_t* pFileName, CTerrain* pTerrain)
{
	m_pTerrain = pTerrain;

	FILE *fp = _wfopen(pFileName, L"rb");
	if(fp == NULL)
	{
		return false;
	}

	// ε
	wchar_t fileInfo[16];
	ZeroMemory(fileInfo, sizeof(fileInfo));
	LoadFileInfo(fp, fileInfo);		// 1. 
	if(0 != wcscmp(L"SOFMAPFILE V1.0", fileInfo))
	{
		fclose(fp);
		return false;
	}

	LoadMapInfo(fp);					// 2. 	
	LoadTileInfo(fp);					// 3. ŸϺ 	
	LoadTexture(fp);					// 4. ؽ 
	LoadAlphaTexture(fp, pFileName);	// 5. ؽ 

	fclose(fp);
	m_pTerrain = NULL;

	return true;
}

void CDecodeSOF::LoadFileInfo(FILE* fp, wchar_t* pFileInfo)
{
	// [1]  
	fread(pFileInfo, 16, 1, fp);
}

void CDecodeSOF::LoadMapInfo(FILE* fp)
{
	// [2]  
	fread(&m_pTerrain->m_MapInfo, sizeof(MAPINFO), 1, fp);
}

void CDecodeSOF::LoadTileInfo(FILE* fp)
{
	// ؽĿ ü ؽ ҷ(y Ʒ )
	m_pTerrain->LoadTerrain();

	int i=0;
	int totalVertex = m_pTerrain->m_MapInfo.nTotalVertices * m_pTerrain->m_MapInfo.nTotalVertices;
	float* temp = new float[totalVertex];

	// [3] ü ؽ y ҷͼ 
	fread(temp, sizeof(float), totalVertex, fp);
	for(i=0; i<totalVertex; ++i)
		m_pTerrain->m_pHeightMap[i].p.y = temp[i];

	delete [] temp;

	// ü 븻 
	POINT pos[3];
	pos[0].x = 0;
	pos[0].y = 0;
	pos[1].x = m_pTerrain->m_MapInfo.nTotalVertices;
	pos[1].y = 0;
	pos[2].x = 0;
	pos[2].y = m_pTerrain->m_MapInfo.nTotalVertices;
	m_pTerrain->ComputeNormal(&pos[0]);

	// Ÿ 
	int totalTile = m_pTerrain->m_MapInfo.nTotalTiles;
	if(m_pTerrain->m_pTileInfo == NULL)
	{
		m_pTerrain->m_pTileInfo = new TILEINFO[totalTile];
	}
	
	for(i=0; i<totalTile; ++i)
	{
		// ŸϺ  (ؽ  )
		m_pTerrain->CreateVB(i, &m_pTerrain->m_pTileInfo[i]);
	}
}

void CDecodeSOF::LoadTexture(FILE* fp)
{
	// [5]  ؽ 
	fread(&m_pTerrain->m_nSelectTextureNum, sizeof(int), 1, fp);

	int i=0;
	for(i=0; i<m_pTerrain->m_nSelectTextureNum; ++i)
	{
		// [6]  ؽ ȣ
		fread(&m_pTerrain->m_nTextureIndex[i], sizeof(int) , 1, fp);
	}	
}

void CDecodeSOF::LoadAlphaTexture(FILE* fp, const wchar_t* pFileName)
{
	int i=0;
	int selectTextureNum = m_pTerrain->m_nSelectTextureNum;
	int alphaSize = m_pTerrain->m_MapInfo.nAlphaTexSize*m_pTerrain->m_MapInfo.nAlphaTexSize;

	BYTE* read = new BYTE[alphaSize];
	for(i=0; i<selectTextureNum-1; ++i)
	{
		// [7] selectTextureNum 2 ̸̻ ؽİ Ѵٴ ̸
		// ؽĸ ϰ ҷ ͸ ؽĿ Ѵ.
		fread(read, sizeof(BYTE), alphaSize, fp);
		m_pTerrain->CreateAlphaTexture(i);
		m_pTerrain->LoadAlphaValue(i, read);
	}
	
	DELS(read);

	// [8] ÷ Ǵ Ÿ  ҷ´.
	int tileNum=0;
	fread(&tileNum, sizeof(int), 1, fp);

	m_pTerrain->m_vecSplattingTile.clear();
	for(i=0; i<tileNum; ++i)
	{
		// [9] ÷ Ǵ Ÿ ȣ ҷ´.
		int tileIndex=0;
		fread(&tileIndex, sizeof(int), 1, fp);

		m_pTerrain->m_vecSplattingTile.push_back(tileIndex);
	}
}