#include "StdAfx.h"
#include "Direct3D_Base.h"

CDirect3D_Base::CDirect3D_Base()
{
	m_pD3D					= NULL;
	m_pDevice				= NULL;
	m_pDevice_Preview		= NULL;
	m_pTex					= NULL;

	::ZeroMemory(m_pPreview, sizeof(m_pPreview));
}

CDirect3D_Base::~CDirect3D_Base()
{
	RELEASE(m_pTex);
	RELEASE(m_pDevice_Preview);
	RELEASE(m_pDevice);
	RELEASE(m_pD3D);
}

// DirectX ʱȭ
HRESULT CDirect3D_Base::Init(HWND hWnd, int width, int height)
{
	if(NULL == (m_pD3D = Direct3DCreate9(D3D_SDK_VERSION)))
	{
		return E_FAIL;
	}

	D3DCAPS9 caps;
	m_pD3D->GetDeviceCaps(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &caps);

	int vp = 0; 
	if(caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) 
		vp = D3DCREATE_HARDWARE_VERTEXPROCESSING;
	else 
		vp = D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	
	D3DDISPLAYMODE d3ddm;
	if(FAILED(m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))
	{
		return E_FAIL;
	}
	
	D3DPRESENT_PARAMETERS d3dpp;	
	ZeroMemory(&d3dpp, sizeof(d3dpp));

	d3dpp.Windowed					= TRUE;
	d3dpp.SwapEffect				= D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat			= d3ddm.Format;
	d3dpp.EnableAutoDepthStencil	= TRUE;
	d3dpp.BackBufferWidth			= width;
    d3dpp.BackBufferHeight			= height;
	d3dpp.AutoDepthStencilFormat	= D3DFMT_D24S8;
	d3dpp.PresentationInterval		= D3DPRESENT_INTERVAL_IMMEDIATE;
	
	if(FAILED(m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
			vp, &d3dpp, &m_pDevice)))
	{
		return E_FAIL;
	}
	
	m_pDevice->SetRenderState(D3DRS_ZENABLE, TRUE);
	m_pDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
	m_pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	// Ʈ, Ʈ, Ȱ
	SetUpMatrix(width, height);
	SetUpLight();
//	SetUpFog(900.0f, 950.0f);

	return S_OK;
}

// Ʈ 
void CDirect3D_Base::SetUpMatrix(int width, int height)
{
	D3DXMATRIX matWorld;	
	D3DXMatrixIdentity(&matWorld);
	m_pDevice->SetTransform(D3DTS_WORLD, &matWorld);
	
	D3DXMATRIX matProj;
	D3DXMatrixPerspectiveFovLH(&matProj,  D3DX_PI/4.0f, (float)width/(float)height, 1.0f, 2000.0f);
	m_pDevice->SetTransform(D3DTS_PROJECTION, &matProj);
}

//  
void CDirect3D_Base::SetUpLight()
{
	D3DMATERIAL9 mtrl;	
	ZeroMemory(&mtrl, sizeof(D3DMATERIAL9));
	mtrl.Ambient	= D3DXCOLOR(1.0f,1.0f,1.0f,1.0f);
	mtrl.Diffuse	= D3DXCOLOR(1.0f,1.0f,1.0f,1.0f);
	mtrl.Specular	= D3DXCOLOR(1.0f,1.0f,1.0f,1.0f);
	m_pDevice->SetMaterial(&mtrl);

	D3DLIGHT9 light;
	ZeroMemory(&light, sizeof(D3DLIGHT9));
	light.Type		= D3DLIGHT_DIRECTIONAL;
	light.Diffuse	= D3DXCOLOR(1.0f,1.0f,1.0f,1.0f);

	D3DXVec3Normalize((D3DXVECTOR3 *)&light.Direction, &D3DXVECTOR3(1.0f, -1.0f, 1.0f));
	m_pDevice->SetLight(0, &light);
	m_pDevice->LightEnable(0, TRUE);
	m_pDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
	m_pDevice->SetRenderState(D3DRS_AMBIENT, 0x00bfbfbf);
}

/*
// Ȱ  
void CDirect3D_Base::SetUpFog(float start, float end, float density)
{
	if(density)
	{
		m_pDevice->SetRenderState(D3DRS_FOGENABLE, TRUE);
		m_pDevice->SetRenderState(D3DRS_FOGCOLOR, COLOR_BG);
		m_pDevice->SetRenderState(D3DRS_FOGSTART, *(DWORD *)&start);
		m_pDevice->SetRenderState(D3DRS_FOGEND, *(DWORD *)&end);
		m_pDevice->SetRenderState(D3DRS_FOGDENSITY, *(DWORD *)&density);
		m_pDevice->SetRenderState(D3DRS_FOGTABLEMODE, D3DFOG_EXP);
	}
	else
	{
		m_pDevice->SetRenderState(D3DRS_FOGENABLE, TRUE);
		m_pDevice->SetRenderState(D3DRS_FOGCOLOR, COLOR_BG);
		m_pDevice->SetRenderState(D3DRS_FOGSTART, *(DWORD *)&start);
		m_pDevice->SetRenderState(D3DRS_FOGEND, *(DWORD *)&end);
		m_pDevice->SetRenderState(D3DRS_FOGVERTEXMODE,D3DFOG_LINEAR);
	}
}
*/

// ֱ  غ
HRESULT CDirect3D_Base::InitPreview(HWND hWnd, int width, int height)
{
	D3DDISPLAYMODE d3ddm;
	if(FAILED(m_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm)))	
	{
		return E_FAIL;
	}

	D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory(&d3dpp, sizeof(d3dpp));

	d3dpp.BackBufferCount			= 1;
    d3dpp.Windowed					= TRUE;
    d3dpp.SwapEffect				= D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferWidth			= width;
    d3dpp.BackBufferHeight			= height;
    d3dpp.BackBufferFormat			= d3ddm.Format;
    d3dpp.hDeviceWindow				= hWnd;
	d3dpp.EnableAutoDepthStencil	= TRUE;
	d3dpp.AutoDepthStencilFormat	= D3DFMT_D24S8;

    // preview
	if(FAILED(m_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &m_pDevice_Preview)))
	{
		return E_FAIL;
	}

	InitPreviewVertex(width, height);

	m_pDevice_Preview->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	m_pDevice_Preview->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	m_pDevice_Preview->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

	m_pDevice_Preview->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
    m_pDevice_Preview->SetRenderState(D3DRS_LIGHTING, FALSE);

	return S_OK;
}

void CDirect3D_Base::ReleasePreview()
{
	RELEASE(m_pTex);
	RELEASE(m_pDevice_Preview);
}

void CDirect3D_Base::InitPreviewVertex(int width, int height)
{
	::ZeroMemory(m_pPreview, sizeof(m_pPreview));

	m_pPreview[0].p = D3DXVECTOR4(0.0f,			0.0f,			0.5f, 1.0f);
    m_pPreview[1].p = D3DXVECTOR4((float)width,	0.0f,			0.5f, 1.0f);
    m_pPreview[2].p = D3DXVECTOR4(0.0f,			(float)height,	0.5f, 1.0f);
    m_pPreview[3].p = D3DXVECTOR4((float)width,	(float)height,	0.5f, 1.0f);

    m_pPreview[0].t.x = 0.0f;
    m_pPreview[0].t.y = 0.0f;
    m_pPreview[1].t.x = 1.0f;
    m_pPreview[1].t.y = 0.0f;
    m_pPreview[2].t.x = 0.0f;
    m_pPreview[2].t.y = 1.0f;
    m_pPreview[3].t.x = 1.0f;
    m_pPreview[3].t.y = 1.0f;
}

HRESULT CDirect3D_Base::LoadPreviewTexture(wchar_t* texName)
{
	RELEASE(m_pTex);

	if(FAILED(D3DXCreateTextureFromFileEx(m_pDevice_Preview, texName, 
				D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN,
				D3DPOOL_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, NULL, NULL, &m_pTex)))
	{
		return E_FAIL;
	}

    return S_OK;
}

// ؽ ׸
void CDirect3D_Base::DrawPreviewTexture()
{
	m_pDevice_Preview->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
	m_pDevice_Preview->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pDevice_Preview->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	m_pDevice_Preview->SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);
	m_pDevice_Preview->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pDevice_Preview->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

	m_pDevice_Preview->SetTexture(0, m_pTex);
	m_pDevice_Preview->SetFVF(PREVIEWVERTEX::FVF);
	m_pDevice_Preview->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, m_pPreview, sizeof(PREVIEWVERTEX));
}

// ٽ 
void CDirect3D_Base::RenderPreview()
{
	if(m_pDevice_Preview)
	{
		m_pDevice_Preview->Clear(0L, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffffffff, 1.0f, 0L);
		if(SUCCEEDED(m_pDevice_Preview->BeginScene()))
		{
			DrawPreviewTexture();
			m_pDevice_Preview->EndScene();
		}

		m_pDevice_Preview->Present(NULL, NULL, NULL, NULL);
	}
}