#include "StdAfx.h"
#include "Input.h"

CInput* g_pInput = NULL;

//-----------------------------------------------------------------------------
// Name:
// Desc: Construction
//-----------------------------------------------------------------------------
CInput::CInput()
{
	m_lpDI			= NULL;
    m_lpKeyboard	= NULL;
    m_lpMouse		= NULL;

	ZeroMemory(&m_MouseState, sizeof(m_MouseState));
	ZeroMemory(&m_KeyState, sizeof(m_KeyState));
}



//-----------------------------------------------------------------------------
// Name:
// Desc: Destruction
//-----------------------------------------------------------------------------
CInput::~CInput()
{
	if(m_lpDI)
    {
		if(m_lpKeyboard)
		{
			m_lpKeyboard->Unacquire();
			m_lpKeyboard->Release();
			m_lpKeyboard = NULL;
		}

		if(m_lpMouse)
		{
			m_lpMouse->Unacquire();
			m_lpMouse->Release();
			m_lpMouse = NULL;
		}
		
		m_lpDI->Release();
		m_lpDI = NULL;
    }
}



//-----------------------------------------------------------------------------
// Name: InitInput()
// Desc: Direct Input ʱȭ(Ű, 콺)
//-----------------------------------------------------------------------------
bool CInput::InitInput(HWND hWnd, HINSTANCE hInst)
{

	//-----------------------------------------------------------------------------
	//	DirectInput 
	//-----------------------------------------------------------------------------
	if(FAILED(DirectInput8Create(hInst, DIRECTINPUT_VERSION,
				IID_IDirectInput8, (void**)&m_lpDI, NULL)))
	{ return false; }
	

	//-----------------------------------------------------------------------------
	//	Keyboard ʱȭ
	//-----------------------------------------------------------------------------
	// Keyboard ̽ 
	if(FAILED(m_lpDI->CreateDevice(GUID_SysKeyboard, &m_lpKeyboard, NULL)))
    {
        MessageBox(NULL, L"keyboard device error.", L"Error", MB_OK);
        return false;
    }

	// Keyboard   
	if(FAILED(m_lpKeyboard->SetCooperativeLevel(hWnd, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE)))
    {
		MessageBox(NULL, L"keyboard SetCooperativeLevel Error.", L"Error", MB_OK);
		return false;
    }

    // Keyboard   .
	if(FAILED(m_lpKeyboard->SetDataFormat(&c_dfDIKeyboard)))
    {
        MessageBox(NULL, L"Keyboard SetDataFormat Error.", L"Error", MB_OK);
        return false;
    }

	// Keyboard ġ . acquire the keyboard
	if(FAILED(m_lpKeyboard->Acquire()))
    {
        MessageBox(NULL, L"Keyboard Acquire Error.", L"Error", MB_OK);
        return false;
    }


	//-----------------------------------------------------------------------------
	//	Mouse ʱȭ
	//-----------------------------------------------------------------------------
    // Mouse ̽ 
	if(FAILED(m_lpDI->CreateDevice(GUID_SysMouse, &m_lpMouse, NULL)))
    {
        MessageBox(NULL, L"mouse device error.", L"Error", MB_OK);
        return false;
    }

	// Mouse   .
	if(FAILED(m_lpMouse->SetCooperativeLevel(hWnd, DISCL_BACKGROUND | DISCL_NONEXCLUSIVE)))
    {
		MessageBox(NULL, L"Mouse SetCooperativeLevel Error.", L"Error", MB_OK);
		return false;
    }

	// Mouse   .
	if(FAILED(m_lpMouse->SetDataFormat(&c_dfDIMouse)))
    {
        MessageBox(NULL, L"Mouse SetDataFormat Error.", L"Error", MB_OK);
        return false;
    }

	// Mouse ġ . acquire the mouse
	if(FAILED(m_lpMouse->Acquire()))
    {
        MessageBox(NULL, L"mouse Acquire Error.", L"Error", MB_OK);
        return false;
    }

	return true;
}



//-----------------------------------------------------------------------------
// Name: Update()
// Desc: Keyboard, Mouse Է° 
//-----------------------------------------------------------------------------
bool CInput::Update()
{
	// Keyboard Է° 
	if(FAILED(m_lpKeyboard->GetDeviceState(sizeof(m_KeyState), (LPVOID)m_KeyState)))
    {
		if(FAILED(m_lpKeyboard->Acquire()))
		{ return false; }

		if(FAILED(m_lpKeyboard->GetDeviceState(sizeof(m_KeyState), (LPVOID)m_KeyState)))
		{ return false; }
    }

	// Mouse Է° 
	if(FAILED(m_lpMouse->GetDeviceState(sizeof(DIMOUSESTATE), &m_MouseState)))
    {
		if(FAILED(m_lpMouse->Acquire()))
		{ return false; }

		if(FAILED(m_lpMouse->GetDeviceState(sizeof(DIMOUSESTATE), &m_MouseState)))
		{ return false; }
    }

    return true;
}



//-----------------------------------------------------------------------------
// Name: KeyDown(), KeyUp()
// Desc: Keyboard , 
//-----------------------------------------------------------------------------
bool CInput::KeyDown(BYTE key)
{
	return (m_KeyState[key] & 0x80) ? true : false;
}
bool CInput::KeyUp(BYTE key)
{
	return (m_KeyState[key] & 0x80) ? false : true;
}



//-----------------------------------------------------------------------------
// Name: ButtonDown(), ButtonUp()
// Desc: Mouse , 
//-----------------------------------------------------------------------------
bool CInput::ButtonDown(int button)
{
	return (m_MouseState.rgbButtons[button] & 0x80) ? true : false;
}
bool CInput::ButtonUp(int button)
{
	return (m_MouseState.rgbButtons[button] & 0x80) ? false : true;
}



//-----------------------------------------------------------------------------
// Name: GetMouseMovement)
// Desc: Mouse ϶
//-----------------------------------------------------------------------------
void CInput::GetMouseMovement(int &dx, int &dy)
{ 
	dx = m_MouseState.lX; dy = m_MouseState.lY;
}



//-----------------------------------------------------------------------------
// Name: GetMouseWheelMovement)
// Desc: Mouse  ϶
//-----------------------------------------------------------------------------
int CInput::GetMouseWheelMovement()
{ 
	return m_MouseState.lZ;
}