#include "StdAfx.h"
#include "MapTool_Mgr.h"
#include "FrameCheck.h"
#include "Terrain.h"
#include "Pick.h"
#include "Input.h"


CMapTool_Mgr g_ToolManager;

CMapTool_Mgr::CMapTool_Mgr()
{
	m_pDevice		= NULL;
	m_pDirect3D		= NULL;
	m_pCamera		= NULL;
	m_pQuadTree		= NULL;
	m_pFont			= NULL;

	m_nWidth  = 0;
	m_nHeight = 0;

	// fps
	m_fps			= 0.0f;
	m_nFrameCount	= 0;
	QueryPerformanceFrequency(&m_frequency);
	QueryPerformanceCounter(&m_startTime);

	// brush
	m_bIsPicking	= TRUE;
	m_bIsBrushInView= TRUE;

	// camera, mouse sensitive
	m_nCameraMove		= 6;

	//  ɼ
	m_bCoordinateAxis	= TRUE;

	// preview
	m_hWnd_Preview		= NULL;
	m_nWidth_Preview	= 0;
	m_nHeight_Preview	= 0;

	m_Particle_Snow = NULL;
}

CMapTool_Mgr::~CMapTool_Mgr()
{
	DEL(m_pDirect3D);
	DEL(g_pTerrain);
	DEL(m_pCamera);
	DEL(g_pInput);
	DEL(m_pQuadTree);
	DEL(m_pFont);
	DEL(g_pPick);
}

// Ŵ ʱȭ
HRESULT	CMapTool_Mgr::Init(HWND hWnd, HINSTANCE hInst, int width, int height)
{
	// 
	m_pDirect3D = new CDirect3D_Base;
	m_nWidth  = width;
	m_nHeight = height;

	if(FAILED(m_pDirect3D->Init(hWnd, width, height)))
	{
		return E_FAIL;
	}
	m_pDevice = m_pDirect3D->GetDevice();

	// ŷŴ 
	g_pPick = new CPick(m_pDevice, hWnd);

	// Ʈ 
	m_pFont = new CD3DFont;
	m_pFont->SetFont(m_pDevice, 8, 16, L"Arial bold");

	return S_OK;
}

// ī޶ 
void CMapTool_Mgr::CameraSetting()
{
	float center = (float)(g_pTerrain->m_MapInfo.nTotalMapSize / 2.0f);
	D3DXVECTOR3 eye(center, 500.0f, -center-300.0f);
	D3DXVECTOR3 look(center, 0.0f, -center);
	D3DXVECTOR3 up(0.0f, 1.0f, 0.0f);

	DEL(m_pCamera);
	m_pCamera = new CCamera(eye, look, up, m_nWidth, m_nHeight);
	
	m_Shader_Water.GetCamera(m_pCamera);
}

void CMapTool_Mgr::DeleteMap()
{
	DEL(g_pTerrain);
	DEL(m_pQuadTree);
	g_pTerrain = new CTerrain(m_pDevice);

	m_Shader_Water.ShutDown();

	delete m_Particle_Snow;
}

//  
void CMapTool_Mgr::CreateMap()
{
	DEL(m_pQuadTree);
	m_pQuadTree = new CQuadTree(g_pTerrain->m_MapInfo.nTiles + 1);
	m_pQuadTree->SetupQuadTree(g_pTerrain);

	int length = g_pTerrain->m_MapInfo.nCellSpacing * g_pTerrain->m_MapInfo.nCells * g_pTerrain->m_MapInfo.nTiles;
	m_Shader_Water.Init(m_pDevice, (float)length, (float)length, 20.0f);
	
	m_Shader_Fog.Init(m_pDevice);

	// create snow
	d3d::BoundingBox boundingBox;
	boundingBox._min = g_Option._Snow.box_min_pt = D3DXVECTOR3(0, 10, -length) ;
	boundingBox._max = g_Option._Snow.box_max_pt = D3DXVECTOR3(length, 500, length) ;
	m_Particle_Snow = new psys::Snow(&boundingBox, g_Option._Snow.number, g_Option._Snow.size);
	m_Particle_Snow->init(m_pDevice, L"Effect/Texture/snowflake.dds");


}

void CMapTool_Mgr::InitTexturePreview()
{
	if(m_pDirect3D)
		m_pDirect3D->InitPreview(m_hWnd_Preview, m_nWidth_Preview, m_nHeight_Preview);
}

void CMapTool_Mgr::UpdateInput(float timeDelta)
{
	if(!g_pInput->Update())
	{ return; }

	int dx, dy, dw;

	m_UserInput.bLButtonDown	= g_pInput->ButtonDown(0);			//콺 ư
	m_UserInput.bRButtonDown	= g_pInput->ButtonDown(1);			//콺 ư
	m_UserInput.bLCtrlKeyDown	= g_pInput->KeyDown(DIK_LCONTROL);	// CTRL ư
	m_UserInput.bLAltKeyDown	= g_pInput->KeyDown(DIK_LALT);		// ALT ư
	m_UserInput.bLShiftKeyDown	= g_pInput->KeyDown(DIK_LSHIFT);	// SHIFT ư

	g_pInput->GetMouseMovement(dx, dy);
	dw = g_pInput->GetMouseWheelMovement();

	// ī޶  1 ~ 10 
	m_UserInput.fMouseMoveDeltaX = (float)(dx*m_UserInput.fMouseSensitive) * (float)m_nCameraMove * timeDelta;
	m_UserInput.fMouseMoveDeltaY = (float)(dy*m_UserInput.fMouseSensitive) * (float)m_nCameraMove * timeDelta;
	m_UserInput.fMouseWheelDelta = (float)(dw*m_UserInput.fMouseSensitive) * 3.0f * timeDelta;
}

void CMapTool_Mgr::UpdateCamera()
{
	// 콺 LBUTTON 
	if(m_UserInput.bLButtonDown)
	{
		// 콺 Viewȿ 鼭 귯  
		if(m_bIsBrushInView && g_pTerrain->m_nBrushType != -1)
		{
			switch(m_nEditMode)
			{
			case EDIT_HEIGHT:
				if(g_pTerrain->m_nBrushType == 0)		// 
				{
					g_pTerrain->EditHeightInCircle();
				}
				else if(g_pTerrain->m_nBrushType == 1)// 簢
				{
					g_pTerrain->EditHeightInRectangle();
				}
				break;

			case EDIT_TEXTURE:
				if(g_pTerrain->m_nTextureSplatting >= 0 &&	// ؽ ϱ or ⸦ ϰ
					g_pTerrain->m_nCurTexIndex >= 1)		// 1~3 ؽĸ Ʈ Ŀ (0 ⺻ؽ )
				{
					g_pTerrain->MakeAlphaTexture(g_pTerrain->m_nCurTexIndex-1, g_pTerrain->m_nBrushType);
				}
				break;

			case EDIT_OBJECT:
				break;
			}
		}
	}

	// 콺 RBUTTON 
	// RButton(̵)
	if(m_UserInput.bRButtonDown && !m_UserInput.bLCtrlKeyDown)
	{
		m_pCamera->MoveCamera(m_UserInput.fMouseMoveDeltaY, m_UserInput.fMouseMoveDeltaX);
	}
	// RButton && Ctrl(ȸ)
	if(m_UserInput.bRButtonDown && m_UserInput.bLCtrlKeyDown)
	{
		m_pCamera->RotateCamera(m_UserInput.fMouseMoveDeltaX, m_UserInput.fMouseMoveDeltaY);
	}

	// 콺 WHEELBUTTON 
	m_pCamera->ZoomCamera(m_UserInput.fMouseWheelDelta);

	m_pDevice->SetTransform(D3DTS_VIEW, &(m_pCamera->GetViewMatrix()));
}

void CMapTool_Mgr::CountFPS()
{
	m_nFrameCount++;
	if(m_nFrameCount > 100)
	{
		LARGE_INTEGER currentTime;

		// ð ´.
		QueryPerformanceCounter(&currentTime);

		float timer = ((float)(currentTime.LowPart - m_startTime.LowPart) / (float)m_frequency.LowPart);
		m_fps = (float)m_nFrameCount / timer;

		m_nFrameCount = 0;
		QueryPerformanceCounter(&m_startTime);
	}
}

// x,y,z   ׷ش.
void CMapTool_Mgr::DrawAxisLine()
{
	m_pDevice->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
	m_pDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_DIFFUSE);
	m_pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);

	AXISLINE line[2];
	line[0].p = m_pCamera->GetLookatPt();
	line[0].p.y = g_pTerrain->GetHeight(line[0].p.x, line[0].p.z) + 1.0f;

	// x
	line[0].color = line[1].color = D3DCOLOR_XRGB(255,0,0);
	line[1].p = line[0].p;
	line[1].p += D3DXVECTOR3(20.0f, 0.0f, 0.0f);
	m_pDevice->SetFVF(AXISLINE::FVF);
	m_pDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, line, sizeof(AXISLINE));

	// y
	line[0].color = line[1].color = D3DCOLOR_XRGB(0,255,0);
	line[1].p = line[0].p;
	line[1].p += D3DXVECTOR3(0.0f, 20.0f, 0.0f);
	m_pDevice->SetFVF(AXISLINE::FVF);
	m_pDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, line, sizeof(AXISLINE));

	// z
	line[0].color = line[1].color = D3DCOLOR_XRGB(0,0,255);
	line[1].p = line[0].p;
	line[1].p += D3DXVECTOR3(0.0f, 0.0f, 20.0f);
	m_pDevice->SetFVF(AXISLINE::FVF);
	m_pDevice->DrawPrimitiveUP(D3DPT_LINELIST, 1, line, sizeof(AXISLINE));

	m_pDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
}

void CMapTool_Mgr::FrameMove(float timeDelta)
{
	if(m_pQuadTree)
	{
		if(!g_pTerrain->m_bFrustumLock)
			g_pTerrain->m_pFrustum->FrustumInit();
		m_pQuadTree->RevealDrawTiles(g_pTerrain);

		// Ʈ 信 ŷüũ 
		if(m_bIsPicking)
		{
			// ȭ 콺  m_bIsBrushInView = false
			if(m_bIsBrushInView = g_pPick->PickInit())
				g_pTerrain->PickTerrain(g_pPick);
		}

		UpdateInput(60.0f/timeDelta);
		UpdateCamera();

		g_pTerrain->SetupLOD(m_pCamera->GetLookatPt());
	
		m_Particle_Snow->update(g_FrameCheck.GetTimeDelta()*0.01);

		CountFPS();
		Render();
	}
}

void CMapTool_Mgr::Render()
{
	m_pDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 
			COLOR_BG, 1.0f, 0);
	
	if(SUCCEEDED(m_pDevice->BeginScene()))
	{

		if(g_Option._Fog._using)
		{
			short a = LOWORD(g_Option._Fog.color);
			short b = HIWORD(g_Option._Fog.color);

			byte _r = LOBYTE(a);
			byte _g = HIBYTE(a);
			byte _b = LOBYTE(b);
			byte _a = HIBYTE(b);

			switch(g_Option._Fog.style)
			{
				case Option_Shader_Fog::Linear:
					m_Shader_Fog.Rander_Begin_LINEAR(m_pCamera->GetEyePt(),
						D3DCOLOR_XRGB(_r,_g,_b), g_Option._Fog.fog_start, g_Option._Fog.fog_end);
				break;

				case Option_Shader_Fog::Exp2:
					m_Shader_Fog.Rander_Begin_Exp2(m_pCamera->GetEyePt(),
						D3DCOLOR_XRGB(100,100,100), g_Option._Fog.exp2, g_Option._Fog.fog_end);
				break;

				case Option_Shader_Fog::Layer:
					m_Shader_Fog.Rander_Begin_Layer(m_pCamera->GetEyePt(),
						D3DCOLOR_XRGB(100,100,100), 200, 100, 1000);
				break;
			}

		}

		{ // draw terrain
			g_pTerrain->DrawTerrain();


			if(m_bIsPicking && m_bIsBrushInView)
			{
				bool bInBrush=false;
				if(m_nEditMode == EDIT_TEXTURE)
					bInBrush = true;

				g_pTerrain->BrushDraw(80, g_pTerrain->m_fBrushOutSize, g_pTerrain->m_fBrushInSize, 
					D3DCOLOR_XRGB(255,0,0), D3DCOLOR_XRGB(255,255,0), bInBrush);
			}

			if(m_bCoordinateAxis)
				DrawAxisLine();

			if(g_pTerrain->m_bFrustumLock)
				g_pTerrain->m_pFrustum->Draw();
		}
		
		if(g_Option._Ocean._using)
			m_Shader_Water.Rander();

		if(g_Option._Fog._using)
			m_Shader_Fog.Rander_End();

		
		if(g_Option._Snow._using)
		{
			m_Particle_Snow->change_Snow_option(g_Option._Snow.number, g_Option._Snow.size, g_Option._Snow.box_min_pt, g_Option._Snow.box_max_pt);
			m_Particle_Snow->render();
		}


		{ // draw info text
			m_pFont->DrawTextEx(10,10,500,30, COLOR_FONT,DT_LEFT, L"FPS : %0.3f", m_fps);
			m_pFont->DrawTextEx(10,30,500,50, COLOR_FONT,DT_LEFT, L"F10 : AlphaTexture Save");
			m_pFont->DrawTextEx(10,50,500,70, COLOR_FONT,DT_LEFT, 
				L"Pick(%0.1f, %0.1f, %0.1f)", g_pTerrain->m_vPickPos.x, g_pTerrain->m_vPickPos.y, g_pTerrain->m_vPickPos.z);

			m_pFont->DrawTextEx(10,70,500,90, COLOR_FONT,DT_LEFT, 
				L"Camera Pos(%0.1f, %0.1f, %0.1f)", g_pPick->m_vPickRayOrig.x, g_pPick->m_vPickRayOrig.y, g_pPick->m_vPickRayOrig.z);

			m_pFont->DrawTextEx(10,90,500,110, COLOR_FONT,DT_LEFT, 
				L"Camera Dist(%0.1f)", m_pCamera->GetDistance());

			m_pFont->DrawTextEx(10,110,500,130, COLOR_FONT,DT_LEFT, 
				L"Visible TileNum(%d)", g_pTerrain->m_vecVisibleTile.size());
		}

		m_pDevice->EndScene();
	}

	m_pDevice->Present(NULL, NULL, NULL, NULL);
}



