#include "StdAfx.h"
#include "QuadTree.h"

CQuadTree::CQuadTree(int tileNum)
{
	// Ʈ 4 ڳ ε ؽ ǥġ 
	m_nEdge[TL]	= 0;
	m_nEdge[TR]	= tileNum - 1;
	m_nEdge[BL]	= tileNum * (tileNum - 1);
	m_nEdge[BR]	= tileNum * tileNum - 1;
	m_nCenter	= (m_nEdge[TL] + m_nEdge[TR] + m_nEdge[BL] + m_nEdge[BR]) / 4;

	m_bCulled	= false;
	m_fRadius	= 0.0f;

	for(int i=0 ; i<4 ; ++i)
	{
		m_pChild[i] = NULL;
	}
}

CQuadTree::CQuadTree(CQuadTree* pParent)
{
	m_nCenter	= 0;
	m_bCulled	= false;
	m_fRadius	= 0.0f;

	for(int i=0 ; i<4 ; ++i)
	{
		m_pChild[i]	= NULL;
		m_nEdge[i]	= 0;
	}
}

CQuadTree::~CQuadTree()
{
	for(int i=0; i<4; ++i)
		delete m_pChild[i];
}

void CQuadTree::SetCorners(int _TL, int _TR, int _BL, int _BR)
{
	m_nEdge[TL]	= _TL;
	m_nEdge[TR]	= _TR;
	m_nEdge[BL]	= _BL;
	m_nEdge[BR]	= _BR;
	m_nCenter	= (m_nEdge[TL] + m_nEdge[TR] + m_nEdge[BL] + m_nEdge[BR]) / 4;
}

CQuadTree* CQuadTree::AddChild(int _TL, int _TR, int _BL, int _BR)
{
	CQuadTree*	pChild;

	pChild = new CQuadTree(this);
	pChild->SetCorners(_TL, _TR, _BL, _BR);
	
	return pChild;
}

bool CQuadTree::SetDivide()
{
	int north	= (m_nEdge[TL] + m_nEdge[TR]) / 2;
	int south	= (m_nEdge[BL] + m_nEdge[BR]) / 2;
	int west	= (m_nEdge[TL] + m_nEdge[BL]) / 2;
	int east	= (m_nEdge[TR] + m_nEdge[BR]) / 2;
	int center	= (m_nEdge[TL] + m_nEdge[TR] + m_nEdge[BL] + m_nEdge[BR]) / 4;

	// ̻  ҰѰ? ׷ٸ SetDivide() 
	if(IsVisible())
	{
		return false;
	}

	// 4 ڽĳ ߰
	m_pChild[TL] = AddChild(m_nEdge[TL], north, west, center);
	m_pChild[TR] = AddChild(north, m_nEdge[TR], center, east);
	m_pChild[BL] = AddChild(west, center, m_nEdge[BL], south);
	m_pChild[BR] = AddChild(center, east, south, m_nEdge[BR]);
	
	return true;
}

void CQuadTree::SetupQuadTree(CTerrain* pTerrain)
{
	if(SetDivide())
	{
		//  2̻  HeightMap ϹǷ ŸϺ Ʈ  쿡  ʴ.
		// ) Ÿ 8*8   1*1϶ ƹ ̻ .
		// ֳϸ Ÿϸ  Ʈ ϱ   1 ص ̻ .
		//   2*2 ϰԵǸ HeightMap ؽ 2 ϴµ
		// Ÿ  Ƿ HeightMap ǥ ã°  ʾ.
		// ׷Ƿ ŸϿ ° Ʈ ϰ   Ѵ.
		int cells = pTerrain->m_MapInfo.nCells;			//  
		int tiles = pTerrain->m_MapInfo.nTiles + 1;		//  Ÿϰ+1 = Ʈ  (2n+1̹Ƿ)
		int vertex= pTerrain->m_MapInfo.nTotalVertices;	//   * Ÿ + 1( ؽ )

		//   ( 2̻  ؼ, 1϶ )
		int ADD_TL = ((m_nEdge[TL]/tiles) * cells*vertex) + ((m_nEdge[TL]%tiles) * cells);
		int ADD_BR = ((m_nEdge[BR]/tiles) * cells*vertex) + ((m_nEdge[BR]%tiles) * cells);

		// ܰ,  ϴ Ÿ Ѵ.
		D3DXVECTOR3 _TL = (D3DXVECTOR3)pTerrain->m_pHeightMap[ADD_TL].p; _TL.y = 0.0f; // y 0.0f
		D3DXVECTOR3 _BR = (D3DXVECTOR3)pTerrain->m_pHeightMap[ADD_BR].p; _BR.y = 0.0f; // y 0.0f
		D3DXVECTOR3 v	= _TL - _BR;

		// v Ÿ  带 δ 豸 ̹Ƿ, 2   Ѵ.
		m_fRadius = D3DXVec3Length(&v) / 2.0f;

		m_pChild[TL]->SetupQuadTree(pTerrain);
		m_pChild[TR]->SetupQuadTree(pTerrain);
		m_pChild[BL]->SetupQuadTree(pTerrain);
		m_pChild[BR]->SetupQuadTree(pTerrain);
	}
}

bool CQuadTree::IsVisible()
{
	return (m_nEdge[TR] - m_nEdge[TL] <= 1);
}

int CQuadTree::IsInFrustum(CTerrain* pTerrain)
{
	bool b[4];
	bool bInSphere;

	int cells = pTerrain->m_MapInfo.nCells;			//  
	int tiles = pTerrain->m_MapInfo.nTiles + 1;		//  Ÿϰ+1 = Ʈ  (2n+1̹Ƿ)
	int vertex= pTerrain->m_MapInfo.nTotalVertices;	//   * Ÿ + 1( ؽ )

	// ּ Ÿ(Ÿũ1) ƴѰ츸 frustum  ˻Ѵ.
	if(!IsVisible())
	{
		//   ( 2̻  ؼ, 1϶ )
		int ADD_CEN = ((m_nCenter/tiles) * cells*vertex) + ((m_nCenter%tiles) * cells);

		// 豸ȿ ִ°?
		bInSphere = pTerrain->m_pFrustum->SphereIsInFrustum(&pTerrain->m_pHeightMap[ADD_CEN].p, m_fRadius);
		if(!bInSphere)
		{
			return ALL_OUT;	// 豸 ȿ    ׽Ʈ 
		}
	}

	//   ( 2̻  ؼ, 1϶ )
	int ADD_TL = ((m_nEdge[TL]/tiles) * cells*vertex) + ((m_nEdge[TL]%tiles) * cells);
	int ADD_TR = ((m_nEdge[TR]/tiles) * cells*vertex) + ((m_nEdge[TR]%tiles) * cells);
	int ADD_BL = ((m_nEdge[BL]/tiles) * cells*vertex) + ((m_nEdge[BL]%tiles) * cells);
	int ADD_BR = ((m_nEdge[BR]/tiles) * cells*vertex) + ((m_nEdge[BR]%tiles) * cells);

	// Ʈ 4   ׽Ʈ
	b[0] = pTerrain->m_pFrustum->VertexIsInFrustum(&pTerrain->m_pHeightMap[ADD_TL].p);
	b[1] = pTerrain->m_pFrustum->VertexIsInFrustum(&pTerrain->m_pHeightMap[ADD_TR].p);
	b[2] = pTerrain->m_pFrustum->VertexIsInFrustum(&pTerrain->m_pHeightMap[ADD_BL].p);
	b[3] = pTerrain->m_pFrustum->VertexIsInFrustum(&pTerrain->m_pHeightMap[ADD_BR].p);

	//  Ǵϱ  2 Ѵ.
	// 1. 4   ȿ (翬^^)
	// 2. leafŸ ϰ ؽ 1 ̸ ̶ 
//	if((b[0] & b[1] & b[2] & b[3]) ||					// 4   ȿ 
//	  (IsVisible() && (b[0] | b[1] | b[2] | b[3])))	// leafŸ ϰ ؽ 1 ̸ ̶ 
	if((b[0] + b[1] + b[2] + b[3]) == 4)
	{
		return ALL_IN;
	}


	// Ϻκ ҿ ִ 
	return PART_IN;
}

void CQuadTree::FrustumCull(CTerrain* pTerrain)
{
	int ret = IsInFrustum(pTerrain);
	switch(ret)
	{
		case ALL_IN :	// frustum ԵǸ ڽĵ鵵  ̹Ƿ  ˻.
			m_bCulled = false;
			return;

		case PART_IN :	// frustum ϺԵǸ ڽĵ鵵 а˻ ʿ
			m_bCulled = false;
			break;

		case ALL_OUT :	// frustum  ڽĵ鵵 Ƿ  ˻.
			m_bCulled = true;
			return;
	}

	// ڽĵ frustum ø
	if(m_pChild[0]) m_pChild[0]->FrustumCull(pTerrain);
	if(m_pChild[1]) m_pChild[1]->FrustumCull(pTerrain);
	if(m_pChild[2]) m_pChild[2]->FrustumCull(pTerrain);
	if(m_pChild[3]) m_pChild[3]->FrustumCull(pTerrain);
}

int CQuadTree::SearchDrawTile(int tileNum, CTerrain* pTerrain)
{
	// ø  ׳ 
	if(m_bCulled)
	{
		m_bCulled = false;
		return tileNum;
	}

	//  尡 µǾ ϴ°?
	// Ÿϰ 1̸ leafŸ̹Ƿ ؾ Ѵ.
	if((m_nEdge[TR] - m_nEdge[TL]) <= 1)
	{
		// ʻ  Ÿ ġ  ִ 
		// ؽ  ŸϺ + 1  ^^;
		// ׷ ŸϹȣ ϱ ؼ Ʒ  Ѵ.
		int tileIndex = m_nEdge[TL] - (m_nEdge[TL] / (pTerrain->m_MapInfo.nTiles+1));

		//   ŸϹȣ Ѵ.(̰͸ ׸ ȴ)
		pTerrain->m_vecVisibleTile.push_back(tileIndex);
		tileNum++;

		return tileNum;
	}

	// ڽ  ˻
	if(m_pChild[TL]) tileNum = m_pChild[TL]->SearchDrawTile(tileNum, pTerrain);
	if(m_pChild[TR]) tileNum = m_pChild[TR]->SearchDrawTile(tileNum, pTerrain);
	if(m_pChild[BL]) tileNum = m_pChild[BL]->SearchDrawTile(tileNum, pTerrain);
	if(m_pChild[BR]) tileNum = m_pChild[BR]->SearchDrawTile(tileNum, pTerrain);

	return tileNum;
}

void CQuadTree::RevealDrawTiles(CTerrain* pTerrain)
{
	// frustum ø Ͽ ׷ Ÿ .
	FrustumCull(pTerrain);

	//  ŸϷ ° Ÿ ׷ ŸϹȣ ˾Ƴ
	// pTerrain->m_vecVisibleTile.push_back(tileIndex) Ѵ.
	pTerrain->m_vecVisibleTile.clear();
	SearchDrawTile(0, pTerrain);
}