#include "stdafx.h"
#include "Shader_Fog.h"

Shader_Fog::Shader_Fog()
{
	m_Effect_Fog   = NULL;
	m_TechHandle = NULL;
}

Shader_Fog::~Shader_Fog()
{
	RELEASE(m_Effect_Fog);
	m_TechHandle = NULL;
}

bool Shader_Fog::Init(LPDIRECT3DDEVICE9 pDevice)
{
	m_pD3DDevice = pDevice;

	// Not using FFP fog
	m_pD3DDevice->SetRenderState(D3DRS_FOGVERTEXMODE, D3DFOG_NONE);
	m_pD3DDevice->SetRenderState(D3DRS_FOGTABLEMODE,  D3DFOG_NONE);
	m_pD3DDevice->SetRenderState(D3DRS_FOGENABLE,FALSE);

	HRESULT hr = 0;
	//
	// Create effect.
	//
	ID3DXBuffer* errorBuffer = 0;
	hr = D3DXCreateEffectFromFile(
		m_pD3DDevice,
		L"Effect/Shader/Fog.fx",
		0,                // no preprocessor definitions
		0,                // no ID3DXInclude interface
		D3DXSHADER_DEBUG, // compile flags
		0,                // don't share parameters
		&m_Effect_Fog,
		&errorBuffer);

	// output any error messages
	if( errorBuffer )
	{
		::MessageBox(0, (wchar_t*)errorBuffer->GetBufferPointer(), 0, 0);
		errorBuffer->Release();

		return false;
	}
	m_TechHandle = m_Effect_Fog->GetTechniqueByName("FOG");

	return true;
}

// ̴ ʿ Ʈ 
void Shader_Fog::SetShaderSetting(D3DXVECTOR3 eyePosition, int Color)
{
	D3DXMATRIX				mat_WorldViewProj;		// WorldViewProjMatrix
	D3DXMATRIX				mat_WorldView;			// WorldView	Matrix
	D3DXMATRIX				mat_World;				// World		Matrix
	D3DXMATRIX				mat_View;					// View			Matrix
	D3DXMATRIX				mat_Proj;					// Projection	Matrix

	m_pD3DDevice->GetTransform(D3DTS_WORLD, &mat_World);
	m_pD3DDevice->GetTransform(D3DTS_VIEW,	&mat_View);
	m_pD3DDevice->GetTransform(D3DTS_PROJECTION, &mat_Proj);

	mat_WorldView = mat_World * mat_View;
	mat_WorldViewProj = mat_World * mat_View * mat_Proj;
	
	D3DXVECTOR4 vCamera(eyePosition, 1.0f);

	short a = LOWORD(Color);
	short b = HIWORD(Color);

	byte _r = LOBYTE(a);
	byte _g = HIBYTE(a);
	byte _b = LOBYTE(b);
	byte _a = HIBYTE(b);

		// fog color  
    D3DXVECTOR4 colFog    = D3DXVECTOR4(_r/255.0f, _g/255.0f, _b/255.0f, _a/255.0f);

	m_Effect_Fog->SetTechnique( m_TechHandle );

	m_Effect_Fog->SetMatrix("matWorldViewProj", &mat_WorldViewProj);
	m_Effect_Fog->SetMatrix("matWorldView", &mat_WorldView);
	m_Effect_Fog->SetMatrix("matWorld", &mat_World);

	m_Effect_Fog->SetVector("vCamera", &vCamera);
	m_Effect_Fog->SetVector("colorFog", &colFog);
}


//   Ȱ
void Shader_Fog::Rander_Begin_LINEAR(D3DXVECTOR3 eyePosition, int Color, float start, float end)
{
	SetShaderSetting(eyePosition, Color);
	m_Effect_Fog->SetInt("Color", Color);
	// ( start /  end  / --   / -- )
	D3DXVECTOR4 fFog      = D3DXVECTOR4(start, end, 0.0f, 0.0f);
	m_Effect_Fog->SetVector("fFog", &fFog);

	UINT numPasses = 0;
	m_Effect_Fog->Begin(&numPasses, 0);

	m_Effect_Fog->BeginPass(0);
}

// ڿα  Ȱ
void Shader_Fog::Rander_Begin_Exp2(D3DXVECTOR3 eyePosition, int Color, float density, float end)
{
	SetShaderSetting(eyePosition, Color);
	// (density/  end / --   / -- )
	D3DXVECTOR4 fFog      = D3DXVECTOR4(density, end, 0.0f, 0.0f);
	m_Effect_Fog->SetVector("fFog", &fFog);

	UINT numPasses = 0;
	m_Effect_Fog->Begin(&numPasses, 0);

	m_Effect_Fog->BeginPass(1);
}


// ̾(ؿ 򸮴) Ȱ
void Shader_Fog::Rander_Begin_Layer(D3DXVECTOR3 eyePosition, int Color, float height, float end, float range)
{
	SetShaderSetting(eyePosition, Color);
	// (height /  end / range   / -- )
	D3DXVECTOR4 fFog      = D3DXVECTOR4(height, end, range, 0.0f);
	m_Effect_Fog->SetVector("fFog", &fFog);

	UINT numPasses = 0;
	m_Effect_Fog->Begin(&numPasses, 0);

	m_Effect_Fog->BeginPass(2);
}

// Ȱȿ 
void Shader_Fog::Rander_End()
{
	m_Effect_Fog->End();
}
