////////////////////////////////////////////////////////////
// 2010 / 02 / 25 : Kim Dong Sung
// Water shader class 
////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Direct3D_Base.h"
#include "FrameCheck.h"
#include "Shader_Water.h" 

void CShader_Water::Init(LPDIRECT3DDEVICE9 d3dDevice, float width, float height, float high)
{
	m_pD3DDevice = d3dDevice;

	m_quadVertices[0].Set(0000.0f,	high,	0000.0f,  0xffffff00,	0.0f,	0.0f);
	m_quadVertices[1].Set(width,	high,	0000.0f,  0xff00ff00,	1.0f,	0.0f);
	m_quadVertices[2].Set(0000.0f,	high,	-height,  0xffff0000,	0.0f,	1.0f);
	m_quadVertices[3].Set(width,	high,	-height,  0xff0000ff,	1.0f,	1.0f);

	m_pD3DDevice->CreateVertexBuffer( 4*sizeof(Vertex), D3DUSAGE_WRITEONLY, 
		Vertex::FVF_Flags, D3DPOOL_DEFAULT, 
		&m_pVertexBuffer, NULL );
	void *pVertices = NULL;

	m_pVertexBuffer->Lock( 0, sizeof(m_quadVertices), (void**)&pVertices, 0 );
	memcpy( pVertices, m_quadVertices, sizeof(m_quadVertices) );
	m_pVertexBuffer->Unlock();

	InitEffect();
}

void CShader_Water::GetCamera(CCamera *pCamera)
{
	m_pCamera = pCamera;
}

void CShader_Water::InitEffect()
{
	//
	// Create a test texture for our effect to use...
	//

	D3DXCreateTextureFromFile( m_pD3DDevice, L"Effect/Texture/waves2.dds", &m_pWaterBump );
	D3DXCreateCubeTextureFromFile( m_pD3DDevice, L"Effect/Texture/cubemap.dds", &m_pReflectionMap );

	HRESULT hr;
	LPD3DXBUFFER pBufferErrors = NULL;

	hr = D3DXCreateEffectFromFile( m_pD3DDevice, 
		                           L"Effect/Shader/water.fx",
		                           NULL, 
		                           NULL, 
		                           D3DXSHADER_ENABLE_BACKWARDS_COMPATIBILITY, 
		                           NULL, 
		                           &m_Effect_Water, 
		                           &pBufferErrors );

	if( FAILED(hr) )
	{
		LPVOID pCompilErrors = pBufferErrors->GetBufferPointer();
		MessageBox(NULL, (const wchar_t*)pCompilErrors, L"Fx Compile Error",
			MB_OK|MB_ICONEXCLAMATION);
	}

}

void CShader_Water::setTechniqueVariables()
{
	//
	// Set up the world matrix for spinning the quad about...
	//
	m_timer += 0.02f;//g_FrameCheck.GetTimeDelta();

	D3DXMATRIX				mat_WorldViewProj;		// WorldViewProjMatrix
	D3DXMATRIX				mat_World;				// World		Matrix
	D3DXMATRIX				mat_View;				// View			Matrix
	D3DXMATRIX				mat_Proj;				// Projection	Matrix

	m_pD3DDevice->GetTransform(D3DTS_WORLD, &mat_World);
	m_pD3DDevice->GetTransform(D3DTS_VIEW,	&mat_View);
	m_pD3DDevice->GetTransform(D3DTS_PROJECTION, &mat_Proj);
	
	//Calculate the matrices
	mat_WorldViewProj = mat_World * mat_View * mat_Proj;
	
	//set the texturs
	m_Effect_Water->SetTexture("texture0", m_pWaterBump);
	m_Effect_Water->SetTexture("texture1", m_pReflectionMap);
	
	//set the matrices
	m_Effect_Water->SetMatrix("WorldViewProj", &mat_WorldViewProj);

	//set eye position
	D3DXVECTOR4 eye(m_pCamera->GetEyePt(), 1.0f);
	m_Effect_Water->SetVector("eyePosition", &eye);
	
	//set the time
	m_Effect_Water->SetFloat("time",m_timer);

	// İ  Ʈ
	m_Effect_Water->SetValue("textureScale",		g_Option._Ocean.textureScale, sizeof(D3DXVECTOR2));
	m_Effect_Water->SetFloat("waveFreq",			g_Option._Ocean.waveFreq);
	m_Effect_Water->SetFloat("waveAmp",				g_Option._Ocean.waveAmp);

	m_Effect_Water->SetFloat("BumpScale",			g_Option._Ocean.BumpScale);

	m_Effect_Water->SetValue("bumpSpeed",			g_Option._Ocean.bumpSpeed, sizeof(D3DXVECTOR2));
	m_Effect_Water->SetVector("deepColor",			&g_Option._Ocean.deepColor);
	m_Effect_Water->SetVector("shallowColor",		&g_Option._Ocean.shallowColor);
	m_Effect_Water->SetFloat("waterAmount",			g_Option._Ocean.waterAmount);

	m_Effect_Water->SetVector("reflectionColor",	&g_Option._Ocean.reflectionColor);
	m_Effect_Water->SetFloat("reflectionAmount",	g_Option._Ocean.reflectionAmount);
	m_Effect_Water->SetFloat("reflectionBlur",		g_Option._Ocean.reflectionBlur);

	m_Effect_Water->SetFloat("fresnelPower",		g_Option._Ocean.fresnelPower);
	m_Effect_Water->SetFloat("fresnelBias",			g_Option._Ocean.fresnelBias);
	m_Effect_Water->SetFloat("hdrMultiplier",		g_Option._Ocean.hdrMultiplier);

}

void CShader_Water::Rander()
{
	if(g_Option._Ocean.AlphaLayer)
	{
		// ؽ  غ. 
		DWORD alpha = 175;
		m_pD3DDevice->SetRenderState(D3DRS_TEXTUREFACTOR, (0x00ffffff)+(alpha<<24));
		m_pD3DDevice->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
		m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);
		m_pD3DDevice->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);

		//  ä ־ ϰ .
		m_pD3DDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
		m_pD3DDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
		m_pD3DDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_TFACTOR );
		m_pD3DDevice->SetRenderState( D3DRS_SRCBLEND , D3DBLEND_SRCALPHA );
		m_pD3DDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
		m_pD3DDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	}

	//
	// Pick one of the techniques in the .fx file to use. For this sample, 
	// we only have one called, "Technique0".
	//

	m_Effect_Water->SetTechnique( "water" );

	setTechniqueVariables();

	// The 0 specifies that ID3DXEffect::Begin and ID3DXEffect::End will 
    // save and restore all state modified by the effect.
	UINT uPasses;
    m_Effect_Water->Begin( &uPasses, 0 );
    
    for( UINT uPass = 0; uPass < uPasses; ++uPass )
    {
        m_Effect_Water->BeginPass( uPass );

		//
		// With each pass, render geometry as you normally would...
		//

        m_pD3DDevice->SetStreamSource( 0, m_pVertexBuffer, 0, sizeof(Vertex) );
		m_pD3DDevice->SetFVF(  Vertex::FVF_Flags );
		m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );

        m_Effect_Water->EndPass();
    }
 
    m_Effect_Water->End();

	if(g_Option._Ocean.AlphaLayer)
		m_pD3DDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
}

void CShader_Water::ShutDown()
{
	RELEASE(m_pVertexBuffer);
	RELEASE(m_pWaterBump);
	RELEASE(m_pReflectionMap);
	RELEASE(m_Effect_Water);
}