////////////////////////////////////////////////////////////
// 2010 / 02 / 25 : Kim Dong Sung
// Water shader class 
////////////////////////////////////////////////////////////

#pragma once

#include <d3d9.h>
#include <d3dx9.h>
#include "Camera.h"		// for get a camera position.

struct Vertex
{
    float x, y, z;
    DWORD color;
    float tu, tv;

    enum FVF
    {
        FVF_Flags = D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1
    };

	void Set(float _x, float _y, float _z, DWORD _color, float _tu, float _tv)
	{
		x = _x;		y = _y;		z = _z;
		color = _color;
		tu = _tu;	tv = _tv;
	}
};

struct Option_Shader_Water
{
	bool							AlphaLayer;
	bool							_using;
	// İ ϴ  
	D3DXVECTOR2						textureScale;
	float							waveFreq;
	float							waveAmp;

	float							BumpScale;
	D3DXVECTOR2						bumpSpeed;
	D3DXVECTOR4						deepColor;
	D3DXVECTOR4						shallowColor;
	float							waterAmount;

	D3DXVECTOR4						reflectionColor;
	float							reflectionAmount;
	float							reflectionBlur;
				
	float							fresnelPower;
	float							fresnelBias;
	float							hdrMultiplier;

};

class CShader_Water
{
public:
	CShader_Water()
	{
		m_pVertexBuffer			= NULL;
		m_pWaterBump			= NULL;
		m_pReflectionMap		= NULL;
		m_Effect_Water				= NULL;
		m_timer = 0.0f;
	}
	
	void GetCamera(CCamera *pCamera);
	void Init(LPDIRECT3DDEVICE9 d3dDevice, float width, float height, float high);
	void InitEffect();

	void setTechniqueVariables();
	void Rander();
	
	void ShutDown();
	
	LPDIRECT3DDEVICE9				m_pD3DDevice;
	LPDIRECT3DVERTEXBUFFER9			m_pVertexBuffer;
	LPDIRECT3DTEXTURE9				m_pWaterBump;			// 
	LPDIRECT3DCUBETEXTURE9			m_pReflectionMap;		// ϴ ݻ 
	LPD3DXEFFECT					m_Effect_Water;
	CCamera							*m_pCamera;				// ī޶ ġ ʿ

	Vertex							m_quadVertices[4];
	float							m_timer;
};