#pragma once

#include <d3d9.h>
#include <d3dx9.h>
#include <vector>

#include "Pick.h"
#include "Frustum.h"

using namespace std;

enum UPDOWNMODE { UP=0, DOWN, FLAT, ORIGIN };
enum LODCRACK	{ LOD_BASIC=0, LOD_LEFT, LOD_TOP, LOD_RIGHT, LOD_BOTTOM };

// 귯
struct BRUSHVERTEX
{
	D3DXVECTOR3 p;
	DWORD color;
	enum { FVF = D3DFVF_XYZ | D3DFVF_DIFFUSE };
};

// 
struct TERRAINVERTEX
{
	D3DXVECTOR3 p;
	D3DXVECTOR3 n;
	D3DXVECTOR2 t1;	// Ÿ uv
	D3DXVECTOR2 t2;	// ؽ uv
};
const DWORD TERRAIN_FVF = (D3DFVF_XYZ | D3DFVF_NORMAL | D3DFVF_TEX2);

// 
// δ ϰ ҷ´.
struct MAPINFO
{
	int nTiles;			//  Ÿϰ(or)
	int nTotalTiles;	// ü Ÿϰ(*)
	int nTotalVertices;	//  ؽ (or)
	int	nTotalMapSize;	// ü  ũ(Ÿ**Ÿ)
	int	nAlphaTexSize;	// ĸ ũ

	int nCells;			// 1Ÿ ȿ ִ 
	int nCellSpacing;	// 1Ÿ ȿ 1 ũ(Ÿ)
	int nVertices;		// 1Ÿ ȿ ִ ؽ  
	int nIndices;		// 1Ÿ ȿ ִ ε  
	int nTriangles;		// 1Ÿ ȿ ִ ﰢ  
	int nTextureNum;	// ü ؽ 
};

// ʾȿ ִ  Ÿ
struct TILEINFO
{
	TERRAINVERTEX*			pVertex;// ŸϺ (ġ,,ؽ)
	LPDIRECT3DVERTEXBUFFER9	pVB;	// pVertex ؽ ۷ 
	int	nEdge[4];					// Ÿ  𼭸(TL, TR, BL, BR) ε(ü ε)
	int nCenter;					// Ÿ  ε(ü ε )
	int	nLodLevel;					// LOD 
	int nLodCrackIndex;				// LOD  
	int	nTriangleNum;				// Ÿ ﰢ 

	TILEINFO(): pVertex(NULL),
				pVB(NULL),
				nCenter(0),
				nLodLevel(0),
				nLodCrackIndex(0),
				nTriangleNum(0)
	{
		int i=0;
		for(i=0; i<4; ++i)
			nEdge[i] = -1;
	}
	~TILEINFO()
	{
		DELS(pVertex);
		RELEASE(pVB);
	}
};

// 簢 ϴ 4
struct EDGEINFO
{
	D3DXVECTOR3 TL;
	D3DXVECTOR3 TR;
	D3DXVECTOR3 BL;
	D3DXVECTOR3 BR;
};

// 4 簢
struct QUADEDGEINFO
{
	EDGEINFO part[4];
};


class CTerrain
{
// 
private:
	LPDIRECT3DDEVICE9		m_pDevice;
	LPDIRECT3DINDEXBUFFER9	(*m_ppIB)[5];
	LPDIRECT3DTEXTURE9		m_pAlphaTexture[3];
	
public:
	LPDIRECT3DTEXTURE9*		m_ppTexture;
	// 
	CFrustum*		m_pFrustum;				// 
	BOOL			m_bFrustumLock;

	//  
	MAPINFO			m_MapInfo;				// 
	TILEINFO*		m_pTileInfo;			// Ÿϵ 
	TERRAINVERTEX*	m_pHeightMap;			// ü 
	BOOL			m_bApplyLOD;			// LOD 뿩
	int				m_nMaxLODLevel;			// LOD ְ 
	int				m_nLODLevelSize;		// LOD  ũ

	// ؽ  
	int				m_nTextureIndex[4];		// 0(⺻)~3 ؽ ȣ(ؽ Ʈ ȣ  ִ)
	int				m_nCurTexIndex;			//  ؽ ȣ(0~3)
	int				m_nSelectTextureNum;	// õ ؽ 
	int				m_nTextureSplatting;	// ÷ ׸ or 

	// 귯 ĥ   ϱ
	vector<int>		m_vecSplattingTile;

	// ؽ ü  
	wstring*			m_strTextureName;	// ü ؽ ̸

	//  Ǵ Ÿ ȣ
	vector<int>		m_vecVisibleTile;	// ̴ ŸϹȣ

	// 귯  1ŸϾ ū ŸϹȣ( ø 귯 ϱ)
	vector<int>		m_vecBrushAreaTile;	// 귯  Ÿ

	//  ɼ
	BOOL			m_bWireFrame;		// ̾  
	BOOL			m_bBoundaryLine;	// 輱 ׸
//	BOOL			m_bFog;				// Ȱ 

	BOOL			m_bLight;			// Ʈ 
	int				m_nUpDownMode;		// ø, , , 
	int				m_nHeightRate;		// 

	// 귯
	int				m_nBrushType;		// , 簢
	float			m_fBrushOutSize;	// ٱ(ε巴)
	float			m_fBrushInSize;		// ʶ(ϰ)
	
	// ŷ
	D3DXVECTOR3		m_vPickPos;			// 콺 ŷǥ
	BOOL			m_bPickOk;			// ŷ 

// Լ
private:
	//  , ü ؽ , ŸϹȣ Ÿ ׸
	void	SetupMapInfo(int tiles, int cells, int cellSpacing, int texNum);
	void	SetupHeightMap();
	void	DrawBaseTile(int index);
	void	DrawSplattingTile(int index);
	void	DrawBoundaryLine(int index);

	/// ī޶Lookġ  Ÿϰ Ÿ  LOD Ѵ.
	inline	void	GetLODLevel(D3DXVECTOR3 look, int index);
	inline	void	SetupCrackIndex(int index);
	
	// ȭ  Լ(̰ , 귯  ׸, 귯  Ȯã(븻))
	inline	float	Lerp(float a, float b, float t);
	inline	void	TexturePaintInBrushArea();
	inline	void	SearchTileInPickArea(POINT pos[3], vector<int>* pVecTile);

	// 귯
	inline	void	BrushCircleDraw(int count, float size, D3DXCOLOR color);
	inline	void	BrushRectangleDraw(int count, float size, D3DXCOLOR color);
	
	// ŷ ȭ  Լ(̴Ÿϰ˻, ŸϹȣ ã, ȣ ã,   ã)
	inline	bool	IsInVisibleTiles(int tileIndex);
	inline	int		SearchPositionInTiles(D3DXVECTOR3 pos, EDGEINFO* pTile=NULL);
	inline	int		SearchPositionInMap(D3DXVECTOR3 pos, EDGEINFO* pMap);
	inline	int		NearPositionInCells(QUADEDGEINFO* pPart, D3DXVECTOR3 pos, EDGEINFO* pCell, int cellIndex);

public:
	CTerrain(LPDIRECT3DDEVICE9 device);
	~CTerrain();

	// ʱȭ 
	HRESULT InitTerrain(int tiles, int cells, int cellSpacing, int texNum, int baseTexIndex);
	HRESULT	CreateVB(int index, TILEINFO* pTileInfo);
	HRESULT	CreateIB(int LODLevel);
	HRESULT	CreateTexture(int index, wchar_t* texName);

	//  ؽ Լ
	HRESULT	CreateAlphaTexture(int index);
	HRESULT	ChangeAlphaTexture(int dest, int src);
	void	SaveAlphaValue(int index, BYTE* pRead);
	void	LoadAlphaValue(int index, BYTE* pRead);
	void	SaveAlphaTexture(int index, const wchar_t* pFileName);

	//  (ҷö,  , 븻)
	void	LoadTerrain();
	float	GetHeight(float x, float z);
	void	ComputeNormal(POINT pos[3]);

	// ׸ Լ
	void	DrawTerrain();
	void	BrushDraw(int count, float outSize, float inSize, D3DXCOLOR outColor, D3DXCOLOR inColor, bool drawInBrush);

	//   ؽ 
	void	EditHeightInCircle();
	void	EditHeightInRectangle();
	void	MakeAlphaTexture(int index, int brushType);

    // LOD
	void	SetupLOD(D3DXVECTOR3 look);

	// ȭ ŷ
	void	PickTerrain(CPick* pPick);

//	void GetCamera(CCamera *pCamera);
};

extern CTerrain *g_pTerrain;